<?php
/**
 * Search criteria for e.g. searching in databases. Do not use this class if
 * you are looking for a ship or company, or what ever. Instead use this class
 * for looking in storages like the database.
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.3.0
 * @copyright	Copyright(c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.mxchange.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class SearchCriteria extends BaseFrameworkSystem implements LocalSearchCriteria {
	/**
	 * Criteria to handle
	 */
	private $searchCriteria = array();

	/**
	 * Limitation for the search
	 */
	private $limit = 0;

	/**
	 * Skip these entries before using them
	 */
	private $skip = 0;

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set part description
		$this->setObjectDescription("Search criteria class");

		// Create unique ID number
		$this->generateUniqueId();

		// Clean up a little
		$this->removeNumberFormaters();
		$this->removeSystemArray();
	}

	/**
	 * Create an instance of this class
	 *
	 * @return	$criteriaInstance	An instance of this criteria
	 */
	public final static function createSearchCriteria () {
		// Get a new instance
		$criteriaInstance = new SearchCriteria();

		// Return this instance
		return $criteriaInstance;
	}

	/**
	 * Add criteria
	 *
	 * @param	$criteriaKey	Criteria key
	 * @param	$criteriaValue	Criteria value
	 * @return	void
	 */
	public function addCriteria ($criteriaKey, $criteriaValue) {
		$this->searchCriteria[$criteriaKey] = $criteriaValue;
	}

	/**
	 * Add configured criteria
	 *
	 * @param	$criteriaKey	Criteria key
	 * @param	$configEntry	Configuration entry
	 * @return	void
	 */
	public function addConfiguredCriteria ($criteriaKey, $configEntry) {
		// Add the configuration entry as a criteria
		$value = $this->getConfigInstance()->readConfig($configEntry);
		$this->addCriteria($criteriaKey, $value);
	}

	/**
	 * Setter for limit
	 *
	 * @param	$limit	Search limit
	 * @return	void
	 */
	public final function setLimit ($limit) {
		$this->limit = $limit;
	}

	/**
	 * Getter for limit
	 *
	 * @param	
	 * @return	$limit	Search limit
	 */
	public final function getLimit () {
		return $this->limit;
	}

	/**
	 * Setter for skip
	 *
	 * @param	$skip	Search skip
	 * @return	void
	 */
	public final function setSkip ($skip) {
		$this->skip = $skip;
	}

	/**
	 * Getter for skip
	 *
	 * @param	
	 * @return	$skip	Search skip
	 */
	public final function getSkip () {
		return $this->skip;
	}

	/**
	 * "Getter" for a cache key
	 *
	 * @return	$cacheKey	The key suitable for the cache system
	 */
	public function getCacheKey () {
		// Initialize the key
		$cacheKey = "";

		// Now walk through all criterias
		foreach ($this->searchCriteria as $criteriaKey => $criteriaValue) {
			// Add the value URL encoded to avoid any trouble with special characters
			$cacheKey .= sprintf("%s=%s;",
				$criteriaKey,
				urlencode($criteriaValue)
			);
		}

		// Add limit and skip values
		$cacheKey .= sprintf("%%limit%%=%s;%%skip%%=%s",
			$this->limit,
			$this->skip
		);

		// Return the cache key
		return $cacheKey;
	}

	/**
	 * Get criteria element or null if not found
	 *
	 * @param	$criteria	The criteria we want to have
	 * @return	$value		Wether the value of the critera or null
	 */
	public function getCriteriaElemnent ($criteria) {
		// Default is not found
		$value = null;

		// Is the criteria there?
		if (isset($this->searchCriteria[$criteria])) {
			// Then use it
			$value = $this->searchCriteria[$criteria];
		}

		// Return the value
		return $value;
	}
}

// [EOF]
?>
