<?php
/**
 * A class for an HTTP response on an HTTP request
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright(c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * The extended headers are taken from phpMyAdmin setup tool, written by
 * Michal Cihar <michal@cihar.com>, licensed under GNU GPL 2.0.
 */
class HttpResponse extends BaseFrameworkSystem implements Responseable {
	/**
	 * Response status
	 */
	private $responseStatus = "200 OK";

	/**
	 * Array with all headers
	 */
	private $responseHeaders = array();

	/**
	 * Body of the response
	 */
	private $responseBody = "";

	/**
	 * Instance of the template engine
	 */
	private $templateEngine = null;

	/**
	 * Fatal resolved messages from filters and so on
	 */
	private $fatalMessages = array();

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set part description
		$this->setObjectDescription("HTTP response");

		// Create unique ID number
		$this->generateUniqueId();

		// Clean up a little
		$this->removeNumberFormaters();
		$this->removeSystemArray();
	}

	/**
	 * Creates an object of this class
	 *
	 * @param	$appInstance		An instance of a manageable application
	 * @return	$responseInstance	A prepared instance of this class
	 */
	public final static function createHttpResponse (ManageableApplication $appInstance) {
		// Get a new instance
		$responseInstance = new HttpResponse();

		// Set the application instance
		$responseInstance->setApplicationInstance($appInstance);

		// Initialize the template engine here
		$responseInstance->initTemplateEngine($appInstance);

		// Return the prepared instance
		return $responseInstance;
	}

	/**
	 * Setter for status
	 *
	 * @param	$status		New response status
	 * @return	void
	 */
	public final function setResponseStatus ($status) {
		$this->responseStatus = (string) $status;
	}

	/**
	 * Add header element
	 *
	 * @param	$name	Name of header element
	 * @param	$value	Value of header element
	 * @return	void
	 */
	public final function addHeader ($name, $value) {
		$this->responseHeaders[$name] = $value;
	}

	/**
	 * Reset the header array
	 *
	 * @return	void
	 */
	public final function resetResponseHeaders () {
		$this->responseHeaders = array();
	}

	/**
	 * "Writes" data to the response body
	 *
	 * @param	$output		Output we shall sent in the HTTP response
	 * @return	void
	 */
	public function writeToBody ($output) {
		$this->responseBody .= $output;
	}

	/**
	 * Sets the response body to something new
	 *
	 * @param	$output		Output we shall sent in the HTTP response
	 * @return	void
	 */
	public function setReponseBody ($output) {
		$this->responseBody = $output;
	}

	/**
	 * Flushs the cached HTTP response to the outer world
	 *
	 * @param	$foce	Wether we shall force the output or abort if headers are
	 *					already sent with an exception
	 * @return	void
	 * @throws	ResponseHeadersAlreadySentException		Thrown if headers are
	 *													already sent
	 */
	public function flushBuffer ($force=false) {
		if ((headers_sent()) && (!$force)) {
			// Headers are already sent!
			throw new ResponseHeadersAlreadySentException($this, self::EXCEPTION_HEADERS_ALREADY_SENT);
		} elseif (!headers_sent()) {
			// Send headers out
			header("HTTP/1.1 {$this->responseStatus}");

			// Used later
			$now = gmdate('D, d M Y H:i:s') . ' GMT';

			// General header for no caching
			$this->addHeader('Expired', $now); // rfc2616 - Section 14.21
			$this->addHeader('Last-Modified', $now);
			$this->addHeader('Cache-Control:', 'no-store, no-cache, must-revalidate, pre-check=0, post-check=0, max-age=0'); // HTTP/1.1
			$this->addHeader('Pragma:', 'no-cache'); // HTTP/1.0

			// Define the charset to be used
			//$this->addHeader('Content-Type:', sprintf("text/html; charset=%s", $this->getConfigInstance()->readConfig('header_charset')));

			foreach ($this->responseHeaders as $name=>$value) {
				header("{$name}: {$value}");
			}
		}

		// Are there some error messages?
		if (count($this->fatalMessages) == 0) {
			// Flush the output to the world
			$this->getWebOutputInstance()->output($this->responseBody);
		} else {
			// Display all error messages
			$this->getApplicationInstance()->handleFatalMessages($this->fatalMessages);

			// Send the error messages out to the world
			$this->getWebOutputInstance()->output($this->responseBody);
		}

		// Clear response header and body
		$this->setReponseBody("");
		$this->resetResponseHeaders();
	}

	/**
	 * Initializes the template engine instance
	 *
	 * @param	$appInstance	An instance of a manageable application
	 * @return	void
	 */
	public final function initTemplateEngine (ManageableApplication $appInstance) {
		$this->templateEngine = $this->prepareTemplateEngine($appInstance);
	}

	/**
	 * Getter for the template engine instance
	 *
	 * @return	$templateEngine		An instance of the used template engine
	 */
	public final function getTemplateEngine () {
		return $this->templateEngine;
	}

	/**
	 * Adds a fatal message id to the response. The added messages can then be
	 * processed and outputed to the world
	 *
	 * @param	$messageId	The message id we shall add
	 * @return	void
	 */
	public final function addFatalMessage ($messageId) {
		// Adds the resolved message id to the fatal message list
		$this->fatalMessages[] = $this->getApplicationInstance()->getLanguageInstance()->getMessage($messageId);
	}
}

// [EOF]
?>
